require 'spec_helper'


OPTS = { :key => 'foobar' }
API_ENDPOINT = "http://foobar:@api.spade.io/v1/"


describe SpadeIO::Client do
  describe "#scrape" do
    it "should fail given bad authentication info" do
      Proc.new do
        SpadeIO::Client.new({})
      end.must_raise RuntimeError
    end

    it "should fail nicely for 300, 400, 500 class errors" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 401, :body =>  file_fixture('empty.json'))

      c = SpadeIO::Client.new(OPTS)
      begin
        res = c.scrape(uri)
      rescue RuntimeError => err
        assert_equal true, !!err.message.match(/.*Unauthorized, forbidden or unacceptable.*/m)
      end
    end
    
    it "should send an authenticated scrape to a page" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 200, :body =>  file_fixture('bigres.json'))

      c = SpadeIO::Client.new(OPTS)
      res = c.scrape(uri)

      expected = JSON.parse(file_fixture('bigres.json')).first['objects']
      assert_equal res, expected

    end

    #XXX DRY the shit out of this fixture code
    it "should populate the result object given a successful response" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 200, :body =>  file_fixture('bigres.json'))

      c = SpadeIO::Client.new(OPTS)
      res = c.scrape(uri)

      expected = JSON.parse(file_fixture('bigres.json')).first['objects']
      assert_equal res, expected
    end

    it "should return nil with an empty response" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 200, :body =>  file_fixture('empty.json'))

      c = SpadeIO::Client.new(OPTS)
      res = c.scrape(uri)

      assert_equal res, nil
    end

    it "should return nil when no objects" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 200, :body =>  file_fixture('no_objects.json'))

      c = SpadeIO::Client.new(OPTS)
      res = c.scrape(uri)

      assert_equal res, nil
    end

    it "should return an error given a failed response" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 500, :body => "")

      c = SpadeIO::Client.new(OPTS)
      Proc.new do
        res = c.scrape(uri)
      end.must_raise RuntimeError
    end

    it "should return sucessfully when given a preview bucket" do
      uri = "http://foo.example.com/article.html"
      stub_request(:get,"#{API_ENDPOINT}scrape?url=#{CGI.escape uri}")
                  .to_return( :status => 200, :body =>  file_fixture('preview_bucket.json'))

      c = SpadeIO::Client.new(OPTS)
      res = c.scrape(uri, 'preview')

      expected = JSON.parse(file_fixture('preview_bucket.json'))
      assert_equal res, expected
    end
    
  end
end
